# -*- coding: utf-8 -*-

import openpyxl

# ---- 起動パラメータ

SRC = 'RG350memo.xlsx'
SheetName = 'Emus'
#Mode = 'md'
Mode = 'wiki'

# ---- 定数

Log = 'log.txt'
Out = 'conv_out.'
Ext = { 'md':'md', 'wiki':'txt' }

Header = [
    '分類',	'名称',	'バージョン名',	'製作',	'リリース日',	'opkID',
    '対象システム名',  'キーバインド：APLメニュー',	'キーバインド：エミュ',
    'ROMファイルディレクトリ',    'ROMファイル形式',	'BIOSディレクトリ',
    'BIOSファイル形式',   '他関連ファイルディレクトリ',	'他関連ファイル',
    '設定ファイルディレクトリ', '設定ファイル',	'セーブディレクトリ',
    'セーブ形式',	'チート設定',    '補足説明',	'特記事項',
    '入手先URL'
    ]


Pghead = { 'md':'# ', 'wiki':'* ' }
Entry = { 'md':'### ', 'wiki':'** ' }
Spec = { 'md':'#### ', 'wiki':'- ' }
Subj = { 'md':'#### ', 'wiki':'*** ' }

# ---- globals

Sheet = ''
items = []

# ----

class Item:
    def __init__(self):
        self.a = {}
        for lbl in Header:
            self.a[lbl] = ''

def getline(row):
    itm = Item()
    for i in range(len(Header)):
       itm.a[Header[i]] = str(row[i].value).replace('\n',' ')

    items.append( itm )
    text = itm.a['分類']+' / '+itm.a['名称']+' / ver='+itm.a['バージョン名']

    return( text )

def toc( idx, dat, mode ):
    form='| {:24s} | {:30s} | {:15s} | {:20s} | {:13s} |\n'
    form1='| [[{:s}]] | {:30s} | {:15s} | {:20s} | {:13s} |\n'
    form2='|~| {:30s} | {:15s} | {:20s} | {:13s} |\n'

    lines = [form.format('Category','Emulator','Version','Target','OPK ID')]
    lines.append( form.format('----','----','----','----','----') )

    cat=''
    for l in idx:
        if mode == 'md':
            lines.append( form.format( dat[l].a['分類'], \
                                   dat[l].a['名称'], \
                                   dat[l].a['バージョン名'], \
                                   dat[l].a['対象システム名'],\
                                   dat[l].a['opkID'][-6:]) )
            cat = dat[l].a['分類']
        elif cat != dat[l].a['分類']:
            lines.append( form1.format( dat[l].a['分類'], \
                                   dat[l].a['名称'], \
                                   dat[l].a['バージョン名'], \
                                   dat[l].a['対象システム名'],\
                                   dat[l].a['opkID'][-6:]) )
            cat = dat[l].a['分類']
        else:
            lines.append( form2.format( \
                                   dat[l].a['名称'], \
                                   dat[l].a['バージョン名'], \
                                   dat[l].a['対象システム名'],\
                                   dat[l].a['opkID'][-6:]) )
    return lines


def formatItem(v, mode):
    buf = [ Entry[mode] + v.a['名称'] + ' : ID='+ v.a['opkID'][-6:] + '\n' ]
    buf.append( Spec[mode]+'Version：{}\n'.format(v.a['バージョン名']) )
    buf.append( Spec[mode]+'対象機種：{}\n'.format(v.a['対象システム名']) )
    buf.append( Spec[mode]+'ファイル形式：{}\n'.format(v.a['ROMファイル形式']) )
    if v.a['BIOSディレクトリ'] != 'None':
        if v.a['BIOSファイル形式'] != 'None':
            buf.append(Spec[mode]+'BIOS：{}&br(){}\n'.\
                       format(v.a['BIOSディレクトリ'], v.a['BIOSファイル形式']) )
        else:
            buf.append( Spec[mode]+'BIOS：{}\n'.format(v.a['BIOSディレクトリ']) )
    if v.a['他関連ファイルディレクトリ'] != 'None':
        buf.append( Subj[mode]+'\t関連ファイル\n{}\n'.format(v.a['他関連ファイルディレクトリ']) )
    if v.a['他関連ファイル'] != 'None':
        buf.append('{}\n'.format(v.a['他関連ファイル']) )
    if v.a['設定ファイルディレクトリ'] != 'None':
        buf.append( Subj[mode]+'\t設定ファイル\n{}\n'.format(v.a['設定ファイルディレクトリ']) )
    if v.a['設定ファイル'] != 'None':
        buf.append('{}\n'.format(v.a['設定ファイル']) )
    if v.a['セーブディレクトリ'] != 'None':
        buf.append( Subj[mode]+'\tセーブファイル\n{}\n'.format(v.a['セーブディレクトリ']) )
    if v.a['セーブ形式'] != 'None':
        buf.append('{}\n'.format(v.a['セーブ形式']) )
    if v.a['チート設定'] != 'None':
        buf.append( Subj[mode]+'チート設定\n{}\n'.format(v.a['チート設定']) )
    if v.a['補足説明'] != 'None':
        buf.append( Subj[mode]+'補足\n{}\n'.format(v.a['補足説明']) )
    if v.a['特記事項'] != 'None':
        buf.append( Subj[mode]+'備考\n{}\n'.format(v.a['特記事項']) )
    if v.a['入手先URL'] != 'None':
        buf.append( Subj[mode]+'リンク\n[[{}]]\n'.format(v.a['入手先URL']) )

    return buf

if __name__ == '__main__':
    wb = openpyxl.load_workbook(SRC)
    Sheet = wb[SheetName]

    of = open(Log, mode = 'w+', encoding='UTF-8')

    po = 0
    sw = 0
    for r in Sheet.rows:
        if r[0].value == Header[0]:
            sw = 1
            continue
        if (r[0].value == None) and (sw == 1):
            break
        if sw == 1:
            text = getline( r )
            of.write( text+'\n' )

    of.close()

    idx = []
    dat = {}
    for v in items:
        ids = '{:15s}{:20s}{:19s}'.format(v.a['分類'][:14],v.a['名称'][:19],v.a['バージョン名'][:19])
        dat[ids] = v
        idx.append( ids )
        print( ids )
    idx.sort()

    of = open(Out+Ext[Mode], mode = 'w+', encoding='UTF-8')
    of.writelines( toc( idx, dat, Mode ) )
    precat = ''
    for l in idx:
        of.write( '\n' )
        if precat != dat[l].a['分類']:
            precat = dat[l].a['分類']
            of.write( Pghead[Mode]+dat[l].a['分類']+'\n' )
        of.writelines( formatItem(dat[l], Mode) )
        of.write( '\n' )
    of.close()
    
